from time import time
from enum import Enum
from typing import Optional, Any, Dict, List

from pydantic import BaseModel, ConfigDict, Field

from phi.model.message import Message, MessageContext


class RunEvent(str, Enum):
    """Events that can be sent by the run() functions"""

    run_started = "RunStarted"
    run_response = "RunResponse"
    run_completed = "RunCompleted"
    tool_call_started = "ToolCallStarted"
    tool_call_completed = "ToolCallCompleted"
    updating_memory = "UpdatingMemory"


class RunResponse(BaseModel):
    """Response returned by Agent.run() or Workflow.run() functions"""

    content: Optional[Any] = None
    content_type: str = "str"
    context: Optional[List[MessageContext]] = None
    event: str = RunEvent.run_response.value
    event_data: Optional[Dict[str, Any]] = None
    messages: Optional[List[Message]] = None
    metrics: Optional[Dict[str, Any]] = None
    model: Optional[str] = None
    run_id: Optional[str] = None
    agent_id: Optional[str] = None
    session_id: Optional[str] = None
    tools: Optional[List[Dict[str, Any]]] = None
    created_at: int = Field(default_factory=lambda: int(time()))

    model_config = ConfigDict(arbitrary_types_allowed=True)

    def get_content_as_string(self, **kwargs) -> str:
        import json

        if isinstance(self.content, str):
            return self.content
        elif isinstance(self.content, BaseModel):
            return self.content.model_dump_json(exclude_none=True, **kwargs)
        else:
            return json.dumps(self.content, **kwargs)



from typing import Union, Dict, List

from httpx import Response

from phi.api.api import api, invalid_response
from phi.api.routes import ApiRoutes
from phi.api.schemas.agent import AgentRunCreate, AgentSessionCreate
from phi.cli.settings import phi_cli_settings
from phi.utils.log import logger


def create_agent_session(session: AgentSessionCreate, monitor: bool = False) -> None:
    if not phi_cli_settings.api_enabled:
        return

    logger.debug("--**-- Logging Agent Session")
    with api.AuthenticatedClient() as api_client:
        try:
            r: Response = api_client.post(
                ApiRoutes.AGENT_SESSION_CREATE if monitor else ApiRoutes.AGENT_TELEMETRY_SESSION_CREATE,
                json={"session": session.model_dump(exclude_none=True)},
            )
            if invalid_response(r):
                logger.debug(f"Invalid response: {r.status_code}, {r.text}")
                return

            response_json: Union[Dict, List] = r.json()
            if response_json is None:
                return

            logger.debug(f"Response: {response_json}")
            return
        except Exception as e:
            logger.debug(f"Could not create Agent session: {e}")
    return


def create_agent_run(run: AgentRunCreate, monitor: bool = False) -> None:
    if not phi_cli_settings.api_enabled:
        return

    logger.debug("--**-- Logging Agent Run")
    with api.AuthenticatedClient() as api_client:
        try:
            r: Response = api_client.post(
                ApiRoutes.AGENT_RUN_CREATE if monitor else ApiRoutes.AGENT_TELEMETRY_RUN_CREATE,
                json={"run": run.model_dump(exclude_none=True)},
            )
            if invalid_response(r):
                logger.debug(f"Invalid response: {r.status_code}, {r.text}")
                return

            response_json: Union[Dict, List] = r.json()
            if response_json is None:
                return

            logger.debug(f"Response: {response_json}")
            return
        except Exception as e:
            logger.debug(f"Could not create Agent run: {e}")
    return



from typing import Union

from fastapi import FastAPI

from phi.api.playground import create_playground_endpoint, PlaygroundEndpointCreate
from phi.utils.log import logger


def serve_playground_app(
    app: Union[str, FastAPI],
    *,
    scheme: str = "http",
    host: str = "localhost",
    port: int = 7777,
    reload: bool = False,
    prefix="/v1",
    **kwargs,
):
    import uvicorn

    try:
        create_playground_endpoint(
            playground=PlaygroundEndpointCreate(
                endpoint=f"{scheme}://{host}:{port}", playground_data={"prefix": prefix}
            ),
        )
    except Exception as e:
        logger.error(f"Could not create playground endpoint: {e}")
        logger.error("Please try again.")
        return

    uvicorn.run(app=app, host=host, port=port, reload=reload, **kwargs)
